@@ -23,7 +23,7 @@ from snakemake.exceptions import WorkflowError
 from snakemake.shell import shell
 from snakemake.common import MIN_PY_VERSION, escape_backslash, SNAKEMAKE_SEARCHPATH
 from snakemake.io import git_content, split_git_path
-from snakemake import singularity
+from snakemake.deployment import singularity
 
 
 PY_VER_RE = re.compile("Python (?P<ver_min>\d+\.\d+).*")
@@ -181,13 +181,13 @@ class Snakemake:
     ):
         # convert input and output to plain strings as some remote objects cannot
         # be pickled
-        self.input = input.plainstrings()
-        self.output = output.plainstrings()
+        self.input = input._plainstrings()
+        self.output = output._plainstrings()
         self.params = params
         self.wildcards = wildcards
         self.threads = threads
         self.resources = resources
-        self.log = log.plainstrings()
+        self.log = log._plainstrings()
         self.config = config
         self.rule = rulename
         self.bench_iteration = bench_iteration
@@ -295,6 +295,7 @@ def script(
     conda_env,
     singularity_img,
     singularity_args,
+    env_modules,
     bench_record,
     jobid,
     bench_iteration,
@@ -398,7 +399,7 @@ def script(
                 REncoder.encode_namedlist(wildcards),
                 threads,
                 REncoder.encode_namedlist(log),
-                REncoder.encode_namedlist(
+                REncoder.encode_dict(
                     {
                         name: value
                         for name, value in resources.items()
@@ -454,7 +455,7 @@ def script(
                     JuliaEncoder.encode_namedlist(wildcards),
                     JuliaEncoder.encode_value(threads),
                     JuliaEncoder.encode_namedlist(log),
-                    JuliaEncoder.encode_namedlist(
+                    JuliaEncoder.encode_dict(
                         {
                             name: value
                             for name, value in resources.items()
@@ -533,6 +534,9 @@ def script(
             if singularity_img is not None:
                 # use python from image
                 py_exec = "python"
+            if env_modules is not None:
+                # use python from environment module
+                py_exec = "python"
             # use the same Python as the running process or the one from the environment
             shell("{py_exec} {f.name:q}", bench_record=bench_record)
         elif language == "r":
@@ -566,5 +570,5 @@ def script(
     finally:
         if f and cleanup_scripts:
             os.remove(f.name)
-        else:
-            logger.warning("Not cleaning up %s" % f.name)
+        elif f:
+            logger.warning("Not cleaning up {} upon request.".format(f.name))
