@@ -49,6 +49,7 @@ from snakemake.io import (
     pipe,
     repeat,
     report,
+    multiext,
     IOFile,
 )
 from snakemake.persistence import Persistence
@@ -81,6 +82,7 @@ class Workflow:
         use_conda=False,
         conda_prefix=None,
         use_singularity=False,
+        use_env_modules=False,
         singularity_prefix=None,
         singularity_args="",
         shadow_prefix=None,
@@ -94,10 +96,18 @@ class Workflow:
         run_local=True,
         default_resources=None,
         cache=None,
+        nodes=1,
+        cores=1,
+        resources=None,
     ):
         """
         Create the controller.
         """
+
+        self.global_resources = dict() if resources is None else resources
+        self.global_resources["_cores"] = cores
+        self.global_resources["_nodes"] = nodes
+
         self._rules = OrderedDict()
         self.first_rule = None
         self._workdir = None
@@ -113,7 +123,6 @@ class Workflow:
         self.included_stack = []
         self.jobscript = jobscript
         self.persistence = None
-        self.global_resources = None
         self.globals = globals()
         self._subworkflows = dict()
         self.overwrite_shellcmd = overwrite_shellcmd
@@ -132,6 +141,7 @@ class Workflow:
         self.use_conda = use_conda
         self.conda_prefix = conda_prefix
         self.use_singularity = use_singularity
+        self.use_env_modules = use_env_modules
         self.singularity_prefix = singularity_prefix
         self.singularity_args = singularity_args
         self.shadow_prefix = shadow_prefix
@@ -239,6 +249,14 @@ class Workflow:
     def rules(self):
         return self._rules.values()
 
+    @property
+    def cores(self):
+        return self.global_resources["_cores"]
+
+    @property
+    def nodes(self):
+        return self.global_resources["_nodes"]
+
     @property
     def concrete_files(self):
         return (
@@ -351,8 +369,6 @@ class Workflow:
         targets=None,
         dryrun=False,
         touch=False,
-        cores=1,
-        nodes=1,
         local_cores=1,
         forcetargets=False,
         forceall=False,
@@ -399,7 +415,6 @@ class Workflow:
         wait_for_files=None,
         nolock=False,
         unlock=False,
-        resources=None,
         notemp=False,
         nodeps=False,
         cleanup_metadata=None,
@@ -426,10 +441,6 @@ class Workflow:
     ):
 
         self.check_localrules()
-
-        self.global_resources = dict() if resources is None else resources
-        self.global_resources["_cores"] = cores
-        self.global_resources["_nodes"] = nodes
         self.immediate_submit = immediate_submit
         self.cleanup_scripts = cleanup_scripts
 
@@ -749,7 +760,7 @@ class Workflow:
         scheduler = JobScheduler(
             self,
             dag,
-            cores,
+            self.cores,
             local_cores=local_cores,
             dryrun=dryrun,
             touch=touch,
@@ -784,17 +795,21 @@ class Workflow:
                 if shell_exec is not None:
                     logger.info("Using shell: {}".format(shell_exec))
                 if cluster or cluster_sync or drmaa:
-                    logger.resources_info("Provided cluster nodes: {}".format(nodes))
+                    logger.resources_info(
+                        "Provided cluster nodes: {}".format(self.nodes)
+                    )
                 else:
                     warning = (
-                        "" if cores > 1 else " (use --cores to define parallelism)"
+                        "" if self.cores > 1 else " (use --cores to define parallelism)"
+                    )
+                    logger.resources_info(
+                        "Provided cores: {}{}".format(self.cores, warning)
                     )
-                    logger.resources_info("Provided cores: {}{}".format(cores, warning))
                     logger.resources_info(
                         "Rules claiming more threads " "will be scaled down."
                     )
 
-                provided_resources = format_resources(resources)
+                provided_resources = format_resources(self.global_resources)
                 if provided_resources:
                     logger.resources_info("Provided resources: " + provided_resources)
 
@@ -986,13 +1001,16 @@ class Workflow:
             if self.default_resources is not None:
                 rule.resources = copy.deepcopy(self.default_resources.parsed)
             if ruleinfo.threads:
-                if not isinstance(ruleinfo.threads, int) and not callable(
-                    ruleinfo.threads
+                if (
+                    not isinstance(ruleinfo.threads, int)
+                    and not isinstance(ruleinfo.threads, float)
+                    and not callable(ruleinfo.threads)
                 ):
                     raise RuleException(
-                        "Threads value has to be an integer or a callable.", rule=rule
+                        "Threads value has to be an integer, float, or a callable.",
+                        rule=rule,
                     )
-                rule.resources["_cores"] = ruleinfo.threads
+                rule.resources["_cores"] = int(ruleinfo.threads)
             if ruleinfo.shadow_depth:
                 if ruleinfo.shadow_depth not in (True, "shallow", "full", "minimal"):
                     raise RuleException(
@@ -1045,40 +1063,59 @@ class Workflow:
                     )
                 # TODO retrieve suitable singularity image
 
-            if ruleinfo.conda_env and self.use_conda:
-                if not (ruleinfo.script or ruleinfo.wrapper or ruleinfo.shellcmd):
-                    raise RuleException(
-                        "Conda environments are only allowed "
-                        "with shell, script, or wrapper directives "
-                        "(not with run).",
-                        rule=rule,
-                    )
-                if not (
-                    urllib.parse.urlparse(ruleinfo.conda_env).scheme
-                    or os.path.isabs(ruleinfo.conda_env)
-                ):
-                    ruleinfo.conda_env = os.path.join(
-                        self.current_basedir, ruleinfo.conda_env
-                    )
-                rule.conda_env = ruleinfo.conda_env
-
-            if self.use_singularity:
+            if self.use_env_modules and ruleinfo.env_modules:
+                # If using environment modules and they are defined for the rule,
+                # ignore conda and singularity directive below.
+                # The reason is that this is likely intended in order to use
+                # a software stack specifically compiled for a particular
+                # HPC cluster.
                 invalid_rule = not (
                     ruleinfo.script or ruleinfo.wrapper or ruleinfo.shellcmd
                 )
-                if ruleinfo.singularity_img:
-                    if invalid_rule:
+                if invalid_rule:
+                    raise RuleException(
+                        "Modules directive is only allowed with "
+                        "shell, script or wrapper directives (not with run)",
+                        rule=rule,
+                    )
+                from snakemake.deployment.env_modules import EnvModules
+
+                rule.env_modules = EnvModules(*ruleinfo.env_modules)
+            else:
+                if ruleinfo.conda_env and self.use_conda:
+                    if not (ruleinfo.script or ruleinfo.wrapper or ruleinfo.shellcmd):
                         raise RuleException(
-                            "Singularity directive is only allowed "
-                            "with shell, script or wrapper directives "
+                            "Conda environments are only allowed "
+                            "with shell, script, or wrapper directives "
                             "(not with run).",
                             rule=rule,
                         )
-                    rule.singularity_img = ruleinfo.singularity_img
-                elif self.global_singularity_img:
-                    if not invalid_rule:
-                        # skip rules with run directive
-                        rule.singularity_img = self.global_singularity_img
+                    if not (
+                        urllib.parse.urlparse(ruleinfo.conda_env).scheme
+                        or os.path.isabs(ruleinfo.conda_env)
+                    ):
+                        ruleinfo.conda_env = os.path.join(
+                            self.current_basedir, ruleinfo.conda_env
+                        )
+                    rule.conda_env = ruleinfo.conda_env
+
+                if self.use_singularity:
+                    invalid_rule = not (
+                        ruleinfo.script or ruleinfo.wrapper or ruleinfo.shellcmd
+                    )
+                    if ruleinfo.singularity_img:
+                        if invalid_rule:
+                            raise RuleException(
+                                "Singularity directive is only allowed "
+                                "with shell, script or wrapper directives "
+                                "(not with run).",
+                                rule=rule,
+                            )
+                        rule.singularity_img = ruleinfo.singularity_img
+                    elif self.global_singularity_img:
+                        if not invalid_rule:
+                            # skip rules with run directive
+                            rule.singularity_img = self.global_singularity_img
 
             rule.norun = ruleinfo.norun
             rule.docstring = ruleinfo.docstring
@@ -1165,6 +1202,13 @@ class Workflow:
 
         return decorate
 
+    def envmodules(self, *env_modules):
+        def decorate(ruleinfo):
+            ruleinfo.env_modules = env_modules
+            return ruleinfo
+
+        return decorate
+
     def global_singularity(self, singularity_img):
         self.global_singularity_img = singularity_img
 
@@ -1272,6 +1316,7 @@ class RuleInfo:
         self.benchmark = None
         self.conda_env = None
         self.singularity_img = None
+        self.env_modules = None
         self.wildcard_constraints = None
         self.threads = None
         self.shadow_depth = None
