@@ -95,7 +95,7 @@ def run(
         # Enforce current workdir (the snakemake source dir) to also be in PYTHONPATH
         # when subprocesses are invoked in the tempdir defined below.
         os.environ["PYTHONPATH"] = os.getcwd()
-    else:
+    elif "PYTHONPATH" in os.environ:
         del os.environ["PYTHONPATH"]
 
     results_dir = join(path, "expected-results")
@@ -172,6 +172,10 @@ def run(
     shutil.rmtree(tmpdir)
 
 
+def test_list_untracked():
+    run(dpath("test_list_untracked"))
+
+
 def test_delete_all_output():
     run(dpath("test_delete_all_output"))
 
@@ -273,10 +277,6 @@ def test_ancient():
     run(dpath("test_ancient"), targets=["D", "old_file"])
 
 
-def test_list_untracked():
-    run(dpath("test_list_untracked"))
-
-
 def test_report():
     run(dpath("test_report"), report="report.html", check_md5=False)
 
@@ -1000,6 +1000,9 @@ def test_pipes2():
     run(dpath("test_pipes2"))
 
 
+@pytest.mark.skip(
+    reason="The AWS Access Key Id you provided does not exist in our records."
+)
 def test_tibanna():
     workdir = dpath("test_tibanna")
     subprocess.check_call(["python", "cleanup.py"], cwd=workdir)
@@ -1099,3 +1102,15 @@ def test_output_file_cache_remote():
         default_remote_provider="S3Mocked",
         default_remote_prefix="test-remote-bucket",
     )
+
+
+def test_multiext():
+    run(dpath("test_multiext"))
+
+
+def test_core_dependent_threads():
+    run(dpath("test_core_dependent_threads"))
+
+
+def test_env_modules():
+    run(dpath("test_env_modules"), use_env_modules=True)
